@@ -139,7 +139,7 @@ class _Project(_Base):
 
     def push_model(self, path, datasource_key, params=None):
         if params is not None:
-            self._make_request('POST', f'/projects/{self.key}/models/{datasource_key}/params', params=params)
+            self._make_request('POST', f'/projects/{self.key}/models/{datasource_key}/params', params={'params': params})
 
         if datasource_key is None:
             url = f'/projects/{self.key}/models'
@@ -196,8 +196,17 @@ class _Datasource(_Base):
         for p in self.dfl.project_participants:
             project_key = p['project_key']
             datasource_key = p['datasource_key']
-            info = self._make_request('GET', f'/projects/{project_key}')
             threads_key = get_threads_key(project_key, datasource_key)
+            '''
+            The first time attach_datasource() we might find 1 previous project for that ds1.
+            So project_participants = [item1]
+            The next time it is called it might also find 1 previous project for ds2
+            So project_participants = [item1, item2]
+            We only want create a thread for items that we haven't done so already
+            '''
+            if threads_key in self.dfl.on_round_threads:
+                continue
+            info = self._make_request('GET', f'/projects/{project_key}')
             self.dfl.on_round_threads[threads_key] = gevent.spawn(self.dfl.train_and_test_callback, datasource_key, info)
 
 
@@ -221,9 +230,7 @@ class DynamoFL(_Base):
             on_error=self._on_error
         )
 
-        t = threading.Thread(target=self.ws.run_forever)
-        t.setDaemon(False)
-        t.start()
+        self.connect_to_ws()
 
     def _on_open(self, ws):
         self.ws.send('{ "action": "auth", "token": "' + self.token + '" }')
@@ -277,8 +284,16 @@ class DynamoFL(_Base):
                 if threads_key in self.on_round_threads:
                     print('Federation error occured:\n  ' + j['data']['errorMessage'])
 
+    def connect_to_ws(self):
+        t = threading.Thread(target=self.ws.run_forever)
+        t.daemon = False
+        t.start()
+
     def _on_close(self, ws, close_status_code, close_msg):
         print('Connection closed')
+        print("Trying again to connect in 30 seconds...")
+        r = threading.Timer(30, self.connect_to_ws)
+        r.start()
 
     def _on_error(self, ws, error):
         print('Connection error:')
